import React, { forwardRef } from 'react'
import PropTypes from 'prop-types'

import ReactEchartsCore from 'echarts-for-react/lib/core'
import echarts from 'echarts/lib/echarts'
import 'echarts/lib/chart/line'
import 'echarts/lib/chart/lines'
import 'echarts/lib/component/legend'
import 'echarts/lib/component/legendScroll'
import 'echarts/lib/component/tooltip'
import 'echarts/lib/component/title'
import 'echarts/lib/component/markPoint'
import 'echarts/lib/component/markLine'
import 'echarts/lib/component/markArea'
import 'echarts/lib/component/dataZoom'

import {
  titleTextStyle,
  lbLegend,
  lbGrid,
  lbXAxis,
  lbYAxis,
  dataZoom,
  labelStyle,
} from '../itemDefaultConfig'

import { formatNumber } from 'ROOT/util/formValidatorRegex'

const AreaChart = forwardRef(
  (
    {
      config: { title, legend, toolbox, grid, xAxis, yAxis, series, ...otherConfig },
      showTitle,
      showLabel,
      rotate,
      interval,
      ...rest
    },
    ref
  ) => {
    console.log('rotate', rotate)
    const option = {
      title: {
        ...titleTextStyle,
        show: showTitle, // title.show优先级高于showTitle
        ...title,
      },
      tooltip: {
        trigger: 'axis',
      },
      legend: {
        top: showTitle ? 27 : 'top',
        ...lbLegend,
        ...legend,
      },
      toolbox: {
        ...toolbox,
      },
      grid: {
        top: showTitle ? 66 : 36,
        ...lbGrid,
        right: '6%',
        ...grid,
      },

      xAxis: {
        axisLabel: {
          interval: interval ? 'auto' : 0,
          rotate: rotate || 0,
        },
        ...lbXAxis,
        // boundaryGap: false,
        ...xAxis,
      },

      yAxis: (yAxis || []).map(item => ({ ...lbYAxis, ...item })),

      series: series || [],
      ...otherConfig,
    }

    let targetArr = []

    option.series.forEach(item => {
      if (showLabel || item.show) {
        item.label = {
          ...labelStyle,
          formatter: function({ value }) {
            // 千分位分隔符
            return `${formatNumber(value)}`
          },
          ...item.label,
        }
      }

      // item.symbol = 'none' // 此处配置 不显示点
      targetArr = targetArr.concat(item.data)
    })

    if (xAxis.data.length > 25) {
      option.dataZoom = dataZoom
    } else {
      option.dataZoom = []
    }

    if (!targetArr.length) {
      // 数据为空时展示
      option.yAxis.max = 5000000
      option.yAxis.min = 0
    } else {
      option.yAxis.max = null
      option.yAxis.min = null
    }

    return <ReactEchartsCore echarts={echarts} notMerge option={option} ref={ref} {...rest} />
  }
)

AreaChart.propTypes = {
  theme: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  interval: PropTypes.bool,
  showTitle: PropTypes.bool,
  showLabel: PropTypes.bool,
  style: PropTypes.object,
  className: PropTypes.string,
  config: PropTypes.object,
  rotate: PropTypes.number,
}

AreaChart.defaultProps = {
  style: {},
  classname: '',
  theme: 'macarons',
  rotate: 0,
  showTitle: false,
  interval: false,
  showLabel: false,
  config: {
    title: {},
    legend: {},
    toolbox: {},
    grid: {},
    xAxis: {
      data: ['周一', '周二', '周三', '周四', '周五', '周六', '周日'],
    },
    yAxis: [{ ...lbYAxis }],
    series: [
      {
        name: '成交',
        type: 'line',

        areaStyle: {},
        data: [10, 12, 21, 54, 260, 830, 710],
      },
      {
        name: '预购',
        type: 'line',
        areaStyle: {},

        data: [30, 182, 434, 791, 390, 30, 10],
      },
      {
        name: '意向',
        areaStyle: {},
        type: 'line',
        data: [1320, 1132, 601, 234, 120, 90, 20],
      },
    ],
  },
}

export default AreaChart
