import React, { forwardRef } from 'react'
import PropTypes from 'prop-types'

import ReactEchartsCore from 'echarts-for-react/lib/core'
import echarts from 'echarts/lib/echarts'
import 'echarts/lib/chart/line'
import 'echarts/lib/chart/lines'
import 'echarts/lib/component/legend'
import 'echarts/lib/component/legendScroll'
import 'echarts/lib/component/tooltip'
import 'echarts/lib/component/title'
import 'echarts/lib/component/markPoint'
import 'echarts/lib/component/markLine'
import 'echarts/lib/component/markArea'
import 'echarts/lib/component/dataZoom'

import {
  titleTextStyle,
  lbLegend,
  lbGrid,
  lbXAxis,
  lbYAxis,
  dataZoom,
  labelStyle,
} from '../itemDefaultConfig'

import { formatNumber } from '../../utils'

const LineChart = forwardRef(
  (
    {
      config: { title, legend, toolbox, grid, xAxis, yAxis, series, ...otherConfig },
      showTitle,
      rotate,
      showLabel,
      interval,
      ...rest
    },
    ref
  ) => {
    const option = {
      title: {
        ...titleTextStyle,
        show: showTitle, // title.show优先级高于showTitle
        ...title,
      },
      tooltip: {
        trigger: 'axis',
      },
      legend: {
        top: showTitle ? 27 : 'top',
        ...lbLegend,
        ...legend,
      },
      toolbox: {
        ...toolbox,
      },
      grid: {
        top: showTitle ? 66 : 36,
        ...lbGrid,
        ...grid,
      },

      xAxis: {
        axisLabel: {
          interval: interval ? 'auto' : 0,
          rotate: rotate || 0,
        },
        ...lbXAxis,
        ...xAxis,
      },

      yAxis: (yAxis || []).map((item) => ({ ...lbYAxis, ...item })),

      series: series || [],
      ...otherConfig,
    }

    let targetArr = []

    option.series.forEach((item) => {
      if (showLabel || item.show) {
        item.label = {
          ...labelStyle,
          formatter: function ({ value }) {
            // 千分位分隔符
            return `${formatNumber(value)}`
          },
          ...item.label,
        }
      }

      targetArr = targetArr.concat(item.data)
    })

    if (xAxis.data.length > 25) {
      option.dataZoom = dataZoom
    } else {
      option.dataZoom = []
    }

    if (!targetArr.length) {
      // 数据为空时展示
      option.yAxis.max = 5000000
      option.yAxis.min = 0
    } else {
      option.yAxis.max = null
      option.yAxis.min = null
    }

    return <ReactEchartsCore echarts={echarts} notMerge option={option} ref={ref} {...rest} />
  }
)

LineChart.propTypes = {
  theme: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  interval: PropTypes.bool,
  showTitle: PropTypes.bool,
  showLabel: PropTypes.bool,
  style: PropTypes.object,
  className: PropTypes.string,
  config: PropTypes.object,
  rotate: PropTypes.number,
}

LineChart.defaultProps = {
  style: {},
  classname: '',
  theme: 'macarons',
  rotate: 0,
  showTitle: false,
  interval: true,
  showLabel: false,
  config: {
    title: {},
    legend: {},
    toolbox: {},
    grid: {},
    xAxis: {
      data: ['示意1', '示意2'],
    },
    yAxis: [{ ...lbYAxis }],
    series: [
      {
        data: [200, 600],
        type: 'line',
        name: '折线图',
      },
    ],
  },
}

export default LineChart
